# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-10-21 12:54
from __future__ import unicode_literals

from django.db import migrations


def __load_data(**kwargs):
    apps = kwargs.pop('apps', None)
    if apps:
        model = apps.get_model("translation_server", "TranslationType")
        try:
            mdl = model.objects.get(tag=kwargs['tag'])
        except model.DoesNotExist:
            mdl = model()
        except Exception as err:
            raise err
        for k, v in kwargs.items():
            setattr(mdl, k, v)
        mdl.save()


def load_data(apps, schema_editor):
    __load_data(apps=apps, tag='DTSM', name_pt_br="Modelo", name_en="Model", has_auxiliary_text=True,
                auxiliary_tag="DTST")
    __load_data(apps=apps, tag='DTSMT', name_pt_br="Meta", name_en="Meta", has_auxiliary_text=True,
                auxiliary_tag="DTSDTSMTP")
    __load_data(apps=apps, tag='DTSG', name_pt_br="Texto", name_en="General Text", has_auxiliary_text=False,
                auxiliary_tag="GTP")
    __load_data(apps=apps, tag='DTSE', name_pt_br="Mensagem de erro", name_en="Error message", has_auxiliary_text=False,
                auxiliary_tag="DTSEE")


def revert_data(apps, schema_editor):
    model = apps.get_model("translation_server", "TranslationType")
    model.objects.filter(tag__in=["DTSM", "DTSMT", "DTSG", "DTSE"]).delete()


class Migration(migrations.Migration):

    dependencies = [
        ('translation_server', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(load_data, revert_data)
    ]



from django.db import models
from django.utils.translation import ugettext_lazy as _


class TranslationType(models.Model):
    created_at = models.DateTimeField(_('DTSM1'), auto_now_add=True, null=True, blank=True, help_text=_('DTST1'))
    updated_at = models.DateTimeField(_('DTSM2'), auto_now=True, null=True, blank=True, help_text=_('DTST2'))
    tag = models.CharField(_('DTSM3'), help_text=_('DTST3'), max_length=20, unique=True)
    name = models.TextField(_('DTSM4'), help_text=_('DTST4'))
    has_auxiliary_text = models.BooleanField(_('DTSM5'), help_text=_('DTST5'), default=True)
    auxiliary_tag = models.CharField(_('DTSM6'), help_text=_('DTST6'), max_length=20, unique=True, blank=True,
                                     default=False, null=True)

    class Meta:
        verbose_name = _('DTSMT1')
        verbose_name_plural = _('DTSMTP1')

    def __str__(self):
        return "%s - %s" % (self.tag, self.name)

    def __unicode__(self):
        return "%s - %s" % (self.tag, self.name)


class Translation(models.Model):
    created_at = models.DateTimeField(_('DTSM1'), auto_now_add=True, null=True, blank=True, help_text=_('DTST1'))
    updated_at = models.DateTimeField(_('DTSM2'), auto_now=True, null=True, blank=True, help_text=_('DTST2'))
    type = models.ForeignKey(TranslationType, on_delete=None, related_name="translation_translation_type",
                             verbose_name=_('DTSM7'), help_text=_('DTST7'))
    tag = models.CharField(_('DTSM8'), help_text=_('DTST8'), max_length=20, unique=True)
    text = models.TextField(_('DTSM9'), help_text=_('DTST9'))
    auxiliary_tag = models.CharField(_('DTSM10'), help_text=_('DTST10'), max_length=20, blank=True, null=True)
    auxiliary_text = models.TextField(_('DTSM11'), help_text=_('DTST11'), blank=True, null=True)
    migration_created = models.BooleanField(_('DTSM12'), help_text=_('DTST12'), default=False)

    class Meta:
        verbose_name = _('DTSMT2')
        verbose_name_plural = _('DTSMTP2')

    def __str__(self):
        return "%s" % self.tag

    def __unicode__(self):
        return "%s" % self.tag


# @receiver(post_save, sender=Translation, dispatch_uid="update_stock_count")
# def update_translation(sender, instance, **kwargs):
#     from django.core.management import call_command
#     call_command('make_translation')

class LastTranslationTag(object):
    translation_tag = None

    def __init__(self, translation_tag, *args, **kwargs):
        self.translation_tag = translation_tag

    def return_last_tag(self):
        if self.translation_tag and len(self.translation_tag) > 0:
            from django.db import connection
            # todo: add suport to multiple databases
            query = "SELECT tag FROM %(translation_table)s WHERE tag LIKE '%(translation_tag)s%%' ORDER BY NULLIF(regexp_replace(TAG, E'\\\\D', '', 'g'), '')::int DESC LIMIT 1" % {
                'translation_tag': self.translation_tag,
                'translation_table': Translation._meta.db_table
            }
            cursor = connection.cursor()
            try:
                cursor.execute(query)
            except Exception as err:
                raise err
            else:
                result = []
                for row in cursor.fetchall():
                    result = row[0]
                if result:
                    import re
                    tag = Translation.objects.get(tag=result)
                    return dict(result=dict(last_tag=result, last_id=re.findall("(\d+)", result)[0], type=tag.type.name,
                                            has_auxiliary_text=tag.type.has_auxiliary_text,
                                            auxiliary_tag=tag.type.auxiliary_tag, tag=tag.type.tag))
                else:
                    return dict(result=dict())
        else:
            return dict(result=dict())



from django.contrib import admin
from modeltranslation.admin import TabbedTranslationAdmin
from django.conf import settings
from translation_server.models import *
from translation_server.forms import *


class CustomModelAdminMixin(object):
    def __init__(self, model, admin_site):
        self.list_display = [field.name for field in model._meta.fields if field.name != "id"]
        super(CustomModelAdminMixin, self).__init__(model, admin_site)
        self.filter_horizontal = [field.name for field in model._meta.get_fields() if
                                  field.many_to_many and "reverse" not in str(type(field))]


@admin.register(TranslationType)
class TranslationTypeAdmin(TabbedTranslationAdmin):
    pass


@admin.register(Translation)
class TranslationAdmin(TabbedTranslationAdmin):
    form = TranslationAdminForm
    fieldsets = (
        ('Translation type', {
            'fields': ('type', 'translations_url', 'translation_type_url', 'last_translation_tag_url')
        }),
        ('Primary info', {
            'fields': ('tag', 'text')
        }),
        ('Auxiliary info', {
            'fields': ('auxiliary_tag', 'auxiliary_text')
        }),
    )
    list_display = ('tag', 'type', 'text')

    class Media:
        import os
        js_dir = os.path.join(settings.STATIC_URL, 'admin/js')
        js = (
            js_dir + '/admin-translation.js',
        )




# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-10-21 12:54
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Translation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, help_text='DTST1', null=True, verbose_name='DTSM1')),
                ('updated_at', models.DateTimeField(auto_now=True, help_text='DTST2', null=True, verbose_name='DTSM2')),
                ('tag', models.CharField(help_text='DTST8', max_length=20, unique=True, verbose_name='DTSM8')),
                ('text', models.TextField(help_text='DTST9', verbose_name='DTSM9')),
                ('text_en', models.TextField(help_text='DTST9', null=True, verbose_name='DTSM9')),
                ('text_pt_br', models.TextField(help_text='DTST9', null=True, verbose_name='DTSM9')),
                ('auxiliary_tag', models.CharField(blank=True, help_text='DTST10', max_length=20, null=True, verbose_name='DTSM10')),
                ('auxiliary_text', models.TextField(blank=True, help_text='DTST11', null=True, verbose_name='DTSM11')),
                ('auxiliary_text_en', models.TextField(blank=True, help_text='DTST11', null=True, verbose_name='DTSM11')),
                ('auxiliary_text_pt_br', models.TextField(blank=True, help_text='DTST11', null=True, verbose_name='DTSM11')),
                ('migration_created', models.BooleanField(default=False, help_text='DTST12', verbose_name='DTSM12')),
            ],
            options={
                'verbose_name': 'DTSMT2',
                'verbose_name_plural': 'DTSMTP2',
            },
        ),
        migrations.CreateModel(
            name='TranslationType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, help_text='DTST1', null=True, verbose_name='DTSM1')),
                ('updated_at', models.DateTimeField(auto_now=True, help_text='DTST2', null=True, verbose_name='DTSM2')),
                ('tag', models.CharField(help_text='DTST3', max_length=20, unique=True, verbose_name='DTSM3')),
                ('name', models.TextField(help_text='DTST4', verbose_name='DTSM4')),
                ('name_en', models.TextField(help_text='DTST4', null=True, verbose_name='DTSM4')),
                ('name_pt_br', models.TextField(help_text='DTST4', null=True, verbose_name='DTSM4')),
                ('has_auxiliary_text', models.BooleanField(default=True, help_text='DTST5', verbose_name='DTSM5')),
                ('auxiliary_tag', models.CharField(blank=True, default=False, help_text='DTST6', max_length=20, null=True, unique=True, verbose_name='DTSM6')),
            ],
            options={
                'verbose_name': 'DTSMT1',
                'verbose_name_plural': 'DTSMTP1',
            },
        ),
        migrations.AddField(
            model_name='translation',
            name='type',
            field=models.ForeignKey(help_text='DTST7', on_delete=django.db.models.deletion.CASCADE, related_name='translation_translation_type', to='translation_server.TranslationType', verbose_name='DTSM7'),
        ),
    ]



from django.test import TestCase
from rest_framework.test import APIRequestFactory
from django.core.urlresolvers import reverse
from rest_framework import status
from rest_framework.test import APITestCase
from translation_server.models import *
from django.urls.exceptions import NoReverseMatch
from django.apps import *
from translation_server.forms import *


class TestLastTranslationTagModel(TestCase):

    def test_translation_tag_is_empty(self):
        """
         If the tag is empty, it should return an empty dict
        """
        last_translation = LastTranslationTag("")
        self.assertEqual(last_translation.return_last_tag(), {'result': {}})

    def test_translation_tag_is_none(self):
        """
         If the tag is None, it should return an empty dict
        """
        last_translation = LastTranslationTag(None)
        self.assertEqual(last_translation.return_last_tag(), {'result': {}})

    def test_translation_tag_exists(self):
        """
         If the tag exists, it should return a dict with the key last_id
        """
        last_translation = LastTranslationTag('DTSM1')
        self.assertIn('last_id', last_translation.return_last_tag()['result'])

    def test_translation_tag_does_not_exists(self):
        """
         If the tag does not exists, it should return an empty dict
        """
        last_translation = LastTranslationTag('DTSM66')
        self.assertNotIn('last_id', last_translation.return_last_tag()['result'])


class TestLastTranslationTagView(APITestCase):

    def test_post_with_id_in_url(self):
        """
        If the method if different from get, it should return error 405
        """
        try:
            url = reverse('get_last_translation_tag', args=[TranslationType.objects.get(tag="DTSM").id])
            print(url)
        except Exception as error:
            raise error
        else:
            response = self.client.post(url, format='json')
            self.assertEqual(response.status_code, status.HTTP_405_METHOD_NOT_ALLOWED)

    def test_get_with_id_in_url(self):
        """
        If the 'tag' is an int, and present in the database, it should return an dict
        """
        try:
            url = reverse('get_last_translation_tag', args=[TranslationType.objects.get(tag="DTSM").id])
        except Exception as error:
            raise error
        else:
            response = self.client.get(url, format='json')
            self.assertEqual(response.status_code, status.HTTP_200_OK)
            self.assertIn("last_id", response.data['result'])

    def test_get_with_empty_tag(self):
        """
        If the 'tag' is empty, it should raise a NoReverseMatch error
        """
        reverse_error = False
        try:
            url = reverse('get_last_translation_tag', args=[""])
        except NoReverseMatch:
            reverse_error = True
        except Exception as error:
            raise error
        else:
            self.assertTrue(reverse_error)

    def test_get_with_tag_equal_none(self):
        """
        If the 'tag' is None, it should return error 404
        """
        reverse_error = False
        try:
            url = reverse('get_last_translation_tag', args=[None])
        except NoReverseMatch:
            reverse_error = True
        except Exception as error:
            raise error
        else:
            response = self.client.get(url, format='json')
            self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
            self.assertFalse(reverse_error)

    def test_get_with_tag_that_exists(self):
        """
        If the 'tag' is present in the database, it should return an dict
        """
        try:
            url = reverse('get_last_translation_tag', args=[TranslationType.objects.get(tag="DTSM").tag])
        except Exception as error:
            raise error
        else:
            response = self.client.get(url, format='json')
            self.assertEqual(response.status_code, status.HTTP_200_OK)
            self.assertIn("last_id", response.data['result'])

    def test_get_with_tag_that_does_not_exist(self):
        """
        If the 'tag' does not exists, it should return error 404
        """
        try:
            url = reverse('get_last_translation_tag', args=["TEST"])
        except Exception as error:
            raise error
        else:
            response = self.client.get(url, format='json')
            self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)


class TestTranslationTypeForm(TestCase):
    def translation_type_creation_without_auxiliary_tag(self):
        """
        the form must be valid without auxiliary tag fields
        """
        fields_to_ignore = ['id', 'created_at', 'updated_at', 'translation_translation_type']
        model_fields = [field.name if field.name not in fields_to_ignore else None for field in
                        apps.get_model('translation_server', "TranslationType")._meta.get_fields()]
        form_data = {}
        for field in model_fields:
            if field:
                if field == 'has_auxiliary_text':
                    value = False
                if field == 'auxiliary_tag':
                    value = ""
                else:
                    value = field.upper()
                form_data.update({field: value})
        form = TranslationTypeAdminForm(data=form_data)
        self.assertTrue(form.is_valid())
        self.assertTrue(form.save())

    def translation_type_creation_with_auxiliary_tag(self):
        """
        the form must be valid with auxiliary tag fields
        """
        fields_to_ignore = ['id', 'created_at', 'updated_at', 'translation_translation_type']
        model_fields = [field.name if field.name not in fields_to_ignore else None for field in
                        apps.get_model('translation_server', "TranslationType")._meta.get_fields()]
        form_data = {}
        for field in model_fields:
            if field:
                if field == 'has_auxiliary_text':
                    value = True
                if field == 'auxiliary_tag':
                    value = "AUX"
                else:
                    value = field.upper()
                form_data.update({field: value})
        form = TranslationTypeAdminForm(data=form_data)
        self.assertTrue(form.is_valid())
        self.assertTrue(form.save())


class TestTranslationForm(TestCase):
    def make_form_data(self, translation_type_tag):
        languages_list = [lang[0].replace('-', '_') for lang in settings.LANGUAGES]
        fields_to_ignore = ['id', 'created_at', 'updated_at', 'translation_translation_type']
        model_fields = [field.name if field.name not in fields_to_ignore else None for field in
                        apps.get_model('translation_server', "Translation")._meta.get_fields()]
        form_data = {}
        translation_type_object = TranslationType.objects.get(tag=translation_type_tag)
        for language in languages_list:
            form_data.update({"text_" + language: "TEST"})
            if translation_type_object.has_auxiliary_text:
                form_data.update({"auxiliary_text_" + language: "TEST"})
        model_fields.pop(model_fields.index("text"))
        model_fields.pop(model_fields.index("auxiliary_text"))
        for field in model_fields:
            if field:
                if field == "type":
                    value = translation_type_object
                if field == 'tag':
                    value = translation_type_object.tag + 1
                if field == "auxiliary_tag":
                    value = translation_type_object.auxiliary_tag + 1 if translation_type_object.has_auxiliary_text else ""
                else:
                    value = field.upper()
                form_data.update({field: value})
        return form_data

    def translation_DTSM_creation(self):
        """
        the form must be valid with DTSM translation type
        """
        form_data = self.make_form_data("DTSM")
        form = TranslationAdminForm(data=form_data)
        self.assertTrue(form.is_valid())
        self.assertTrue(form.save())

    def translation_DTSMT_creation(self):
        """
        the form must be valid with DTSMT translation type
        """
        form_data = self.make_form_data("DTSMT")
        form = TranslationAdminForm(data=form_data)
        self.assertTrue(form.is_valid())
        self.assertTrue(form.save())

    def translation_DTSE_creation(self):
        """
        the form must be valid with DTSE translation type
        """
        form_data = self.make_form_data("DTSE")
        form = TranslationAdminForm(data=form_data)
        self.assertTrue(form.is_valid())
        self.assertTrue(form.save())

    def translation_DTSG_creation(self):
        """
        the form must be valid with DTSG translation type
        """
        form_data = self.make_form_data("DTSG")
        form = TranslationAdminForm(data=form_data)
        self.assertTrue(form.is_valid())
        self.assertTrue(form.save())



from django.shortcuts import render
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext as __
from django.conf import settings
from translation_server.models import *
from translation_server.serializers import *
from rest_framework import viewsets
from rest_framework import filters
from rest_framework import views
from rest_framework.response import Response
from rest_framework import status
from datetime import datetime
from django.http import JsonResponse, HttpResponse
from django.template.loader import get_template
from django.core.mail import EmailMessage
from django.template import Context
from rest_framework.permissions import AllowAny


class TranslationViewSet(viewsets.ReadOnlyModelViewSet):
    """
    This viewset automatically provides `list` and `detail` actions.
    """
    queryset = Translation.objects.all()
    serializer_class = TranslationSerializer
    filter_backends = (filters.DjangoFilterBackend,)
    filter_fields = ('id', 'tag', 'type')


class TranslationTypeViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = TranslationType.objects.all()
    serializer_class = TranslationTypeSerializer
    filter_backends = (filters.DjangoFilterBackend,)
    filter_fields = ('id', 'tag')


class LastTranslationTagView(views.APIView):
    def get(self, request, *args, **kwargs):
        if len(kwargs['tag']) > 0:
            try:
                int(kwargs['tag'])
            except ValueError:
                try:
                    model = TranslationType.objects.get(tag=kwargs['tag'])
                except TranslationType.DoesNotExist:
                    return Response(status=status.HTTP_404_NOT_FOUND)
            else:
                try:
                    model = TranslationType.objects.get(pk=kwargs['tag'])
                except TranslationType.DoesNotExist:
                    return Response(status=status.HTTP_404_NOT_FOUND)
            last_translation_tag = LastTranslationTag(model.tag, *args, **kwargs)
            result = last_translation_tag.return_last_tag()
        else:
            result = dict(result=dict())
        return Response(result, status=status.HTTP_200_OK)